import { Mapper } from '@automapper/core';
import { InjectMapper } from '@automapper/nestjs';
import { CommandHandler } from '@nestjs/cqrs';
import { TerritoriesRepository } from '../../adapters/secondaries/territories.repository';
import { Messager } from '../../adapters/secondaries/messager';
import { TerritoryLoggingPresenter } from '../../adapters/secondaries/territory-logging.presenter';
import { UpdateTerritoryCommand } from '../../commands/update-territory.command';
import { UpdateTerritoryRequest } from '../dtos/update-territory.request';
import { Territory } from '../entities/territory';
import { TerritoryPresenter } from '../../adapters/secondaries/territory.presenter';

@CommandHandler(UpdateTerritoryCommand)
export class UpdateTerritoryUseCase {
  constructor(
    private readonly _repository: TerritoriesRepository,
    private readonly _messager: Messager,
    @InjectMapper() private readonly _mapper: Mapper,
  ) {}

  async execute(command: UpdateTerritoryCommand): Promise<Territory> {
    try {
      const territory = await this._repository.updateTerritory(
        command.updateTerritoryRequest.uuid,
        this._mapper.map(
          command.updateTerritoryRequest,
          UpdateTerritoryRequest,
          Territory,
        ),
      );
      this._messager.publish(
        'territory.update',
        JSON.stringify(
          this._mapper.map(
            command.updateTerritoryRequest,
            UpdateTerritoryRequest,
            TerritoryPresenter,
          ),
        ),
      );
      this._messager.publish(
        'logging.territory.update.info',
        JSON.stringify(
          this._mapper.map(
            command.updateTerritoryRequest,
            UpdateTerritoryRequest,
            TerritoryLoggingPresenter,
          ),
        ),
      );
      return territory;
    } catch (error) {
      this._messager.publish(
        'logging.territory.update.crit',
        JSON.stringify({
          command,
          error,
        }),
      );
      throw error;
    }
  }
}
