import { Test, TestingModule } from '@nestjs/testing';
import { TerritoriesRepository } from '../../adapters/secondaries/territories.repository';
import { FindAllTerritoriesForPointsRequest } from '../../domain/dtos/find-all-territories-for-points.request';
import { Point } from '../../domain/entities/point';
import { FindAllTerritoriesForPointsUseCase } from '../../domain/usecases/find-all-territories-for-points.usecase';
import { FindAllTerritoriesForPointsQuery } from '../../queries/find-all-territories-for-points.query';

const findAllTerritoriesForPointsRequest: FindAllTerritoriesForPointsRequest =
  new FindAllTerritoriesForPointsRequest();
findAllTerritoriesForPointsRequest.points = [
  new Point(6.181455, 48.685689),
  new Point(6.191455, 48.695689),
];

const findforPointsQuery: FindAllTerritoriesForPointsQuery =
  new FindAllTerritoriesForPointsQuery(findAllTerritoriesForPointsRequest);

const mockTerritories = [
  {
    uuid: 'bb281075-1b98-4456-89d6-c643d3044a91',
    name: 'Nancy',
  },
  {
    uuid: 'bb281075-1b98-4456-89d6-c643d3044a92',
    name: 'Meurthe-et-Moselle',
  },
];

const mockTerritoriesRepository = {
  findForPoints: jest
    .fn()
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    .mockImplementation((query?: FindAllTerritoriesForPointsQuery) => {
      return Promise.resolve(mockTerritories);
    }),
};

describe('FindAllTerritoriesforPointsUseCase', () => {
  let findAllTerritoriesForPointsUseCase: FindAllTerritoriesForPointsUseCase;

  beforeAll(async () => {
    const module: TestingModule = await Test.createTestingModule({
      providers: [
        {
          provide: TerritoriesRepository,
          useValue: mockTerritoriesRepository,
        },
        FindAllTerritoriesForPointsUseCase,
      ],
    }).compile();

    findAllTerritoriesForPointsUseCase =
      module.get<FindAllTerritoriesForPointsUseCase>(
        FindAllTerritoriesForPointsUseCase,
      );
  });

  it('should be defined', () => {
    expect(findAllTerritoriesForPointsUseCase).toBeDefined();
  });

  describe('execute', () => {
    it('should return an array filled with territories', async () => {
      const territories = await findAllTerritoriesForPointsUseCase.execute(
        findforPointsQuery,
      );

      expect(territories).toBe(mockTerritories);
    });
  });
});
